#Association networks in urban and natural space

rm(list=ls())

library(spatsoc)
library(lubridate)
library(data.table)
library(plyr)
library(asnipe)
library(igraph)
library(ggplot2)

# Load in dataframe

setwd()
load("GPS.RData")

GPS <- GPS[GPS$activecollars >= 10,]
GPS <- GPS[GPS$activecollars_area == GPS$activecollars,]
GPS$hour <- substr(GPS$time, 12, 13)
GPS <- GPS[GPS$hour < 16,]

GPS$time <- as.POSIXct(GPS$time, "%Y-%m-%d %H:%M:%S", tz = "UTC")
GPS$min <- floor_date(GPS$time, unit="minute")
GPS$date <- as.Date(GPS$time)

GPSmin <- ddply(.data = GPS, 
               .variables = c("id"),  
               .fun = function(x) { 
                 min <- x[!duplicated(x$min),]
                 dat = data.frame(min) 
                 return(dat) 
               },
               .progress = "text") 

GPSmin <- GPSmin[c("id","min","x","y","area")]

DT <- setDT(GPSmin)

DT[, time := as.POSIXct(min, tz = 'UTC')]

group_times(DT, datetime = 'time', threshold = '1 minute')

group_pts(
  DT,
  threshold = 5,
  id = 'id',
  coords = c('x', 'y'),
  timegroup = 'timegroup',
  splitBy = 'area'
)

Natural <- DT[area == "Natural"]

gbiMtrx_natural <- get_gbi(DT = Natural, group = 'group', id = 'id')

net_natural <- get_network(gbiMtrx_natural,
                           data_format = "GBI",
                           association_index = "SRI")

g_natural <- graph.adjacency(net_natural, 'undirected', 
                             diag = FALSE, weighted = TRUE)

natural_assocNetwork <- data.table(
  centrality = evcent(g_natural)$vector,
  strength = graph.strength(g_natural),
  ID = names(degree(g_natural))
)

Urban <- DT[area == "Urban"]

gbiMtrx_urban <- get_gbi(DT = Urban, group = 'group', id = 'id')

net_urban <- get_network(gbiMtrx_urban,
                         data_format = "GBI",
                         association_index = "SRI")

g_urban <- graph.adjacency(net_urban, 'undirected', 
                           diag = FALSE, weighted = TRUE)


urban_assocNetwork <- data.table(
  centrality = evcent(g_urban)$vector,
  strength = graph.strength(g_urban),
  ID = names(degree(g_urban))
)

DF <- rbind(natural_assocNetwork, urban_assocNetwork)

AssocNetwork <- as.data.frame(DF)
head(AssocNetwork)

colnames(AssocNetwork) <- c("AssocCentrality", "AssocStrength", "ID")

coNat <- cluster_walktrap(g_natural)
modularity(coNat)

coUrb <- cluster_walktrap(g_urban)
modularity(coUrb)

edge_density(g_natural)
edge_density(g_urban)
cor.test(c(as.matrix(net_natural)), c(as.matrix(net_urban)), method = "spearman")

### Subgroups in 150 m

DT <- setDT(GPSmin)

DT[, time := as.POSIXct(min, tz = 'UTC')]

group_times(DT, datetime = 'time', threshold = '1 minute')

group_pts(
  DT,
  threshold = 150,
  id = 'id',
  coords = c('x', 'y'),
  timegroup = 'timegroup',
  splitBy = 'area'
)

DT <- as.data.frame(DT)

df <-
  ddply(DT, .(group), summarize,
        In150 = length(group))

DT <- DT[!duplicated(DT$group),]

df <- merge(df, DT[c("group", "area")], by = "group", all.x=T)

p1 <-ggplot(df, aes(x=In150, fill=area)) +
  geom_density(alpha=0.4)+ 
  scale_fill_manual(values=c("#008000","#696969"), labels = c("Natural", "Urban")) + 
  ylab("Density") + xlab("No. of individuals, in 150m")

p1
