rm(list = ls())

# load packages

library(plyr)
library(data.table)
library(igraph)
library(network)

#Functions to extract pulls and anchors (from Strandburg-Peshkin et al. 2015)

get.dyad.interactions <- function(xa,xb,ya,yb,a,b, noise.thresh = 5, plot.results = F){
  
  #get the number of imtes
  n.times <- length(xa)
  
  #check to make sure xa, xb, ya, and yb all contain the same number of elements
  if(length(xb) != n.times | length(ya) != n.times | length(yb) != n.times){stop('ERROR: x and y data vectors are not all the same length')}
  
  #check to make sure data streams are continuous
  if(any(is.na(c(xa,xb,ya,yb)))){stop('ERROR: x and/or y data contain NAs for the specified individuals')}
  
  #get dyadic distances over time
  dyad.dist <- sqrt((xa-xb)^2 + (ya-yb)^2)
  
  #---find first minimum---
  #get first instance of a change in dyadic distance above noise.thresh, and determine whether it is going up or down
  i <- 1
  curr.min <- 1
  found = F
  while(found == F & i < length(dyad.dist)){
    dist.change <- dyad.dist[i] - dyad.dist[1]
    if(dyad.dist[i]<dyad.dist[curr.min]){
      curr.min <- i
    }
    if(abs(dist.change)>noise.thresh){
      found = T
      if(dist.change > 0){
        up.first <- T
      }
      else{
        up.first <- F
      }
    }
    else{
      i <- i + 1
    }
  }
  
  #if no first minimum found, return NULL
  if(!found){
    return(NULL)
  }
  
  #find starting point (if it went up first, this is curr.min, otherwise, this is the first minimum)
  if(up.first){
    first <- curr.min
  }
  else{
    curr.min <- 1
    found <- F
    while(i < n.times & !found){
      dist.diff <- dyad.dist[i] - dyad.dist[curr.min]
      if(dist.diff < 0){
        curr.min <- i
      }
      else{
        if(dist.diff > noise.thresh){
          found <- T
          first <- curr.min
        }
      }
      i<-i+1
    }
    if(!found){ #if no starting point found, return NULL
      return(NULL)
    }
  }
  
  #----pull out maxes and mins in dyadic distance----
  min.max.min <- data.frame(t1 = NA, t2 = NA, t3 = NA)
  min.max.min$t1[1] <- first
  ref.idx <- first
  ref.val <- dyad.dist[first]
  data.idx <- 1
  going.up <- T
  for(i in first:n.times){
    curr.idx <- i
    curr.val <- dyad.dist[i]
    ref.val <- dyad.dist[ref.idx] #NEW
    if(going.up){ #if going up
      if(curr.val > ref.val){ #if the current value is greater than the reference value
        ref.idx <- curr.idx #replace the reference value with the current value
        ref.val <- dyad.dist[ref.idx]
      }
      else{ #if not
        if(abs(curr.val - ref.val) > noise.thresh){ #if the difference between current and reference exceeds the noise threshold
          min.max.min$t2[data.idx] <- ref.idx #record the local max
          going.up <- F #switch to going down
          ref.idx <- i #NEW - ref index becomes the current index
        } 
      }
    }
    else{ #if going down
      if(curr.val < ref.val){ #if the current value is less than the reference value
        ref.idx <- curr.idx #replace the reference value with the current value
        ref.val <- dyad.dist[ref.idx]
      }
      else{ #if not
        if(abs(curr.val - ref.val) > noise.thresh){ #if the difference between current and reference exceeds a noise threshold
          min.max.min$t3[data.idx] <- ref.idx #record the local min as the end of the current sequence
          data.idx <- data.idx + 1 #increment the data index
          min.max.min <- rbind(min.max.min,c(ref.idx,NA,NA)) #record the local min as the start of the next sequence #USED TO BE REF.IDX
          going.up <- T #switch to going up
          ref.idx <- i #NEW - ref.idx becomes the current index
        }	
      }
    }
  }
  
  if(plot.results){
    plot(dyad.dist)
    for(i in 1:nrow(min.max.min)){
      abline(v=min.max.min$t1[i],col='red')
      abline(v=min.max.min$t2[i],col='green')
      abline(v=min.max.min$t3[i],col='red')
    }
  }
  
  #remove the last row if it is incomplete
  min.max.min <- min.max.min[which(!is.na(min.max.min$t3)),]
  
  #if no min.max.min sequences were found, return NULL
  if(nrow(min.max.min)==0){
    return(NULL)
  }
  
  #---determine whether they are successful pulls ('pull') or failed pulls ('anchor') and who is leading (attempting to pull)
  
  #get displacements for each individual during t1 to t2 (1) and t2 to t3 (2)
  min.max.min$disp.a.1 <- sqrt((xa[min.max.min$t2] - xa[min.max.min$t1])^2 + (ya[min.max.min$t2] - ya[min.max.min$t1])^2)
  min.max.min$disp.a.2 <- sqrt((xa[min.max.min$t3] - xa[min.max.min$t2])^2 + (ya[min.max.min$t3] - ya[min.max.min$t2])^2)
  min.max.min$disp.b.1 <- sqrt((xb[min.max.min$t2] - xb[min.max.min$t1])^2 + (yb[min.max.min$t2] - yb[min.max.min$t1])^2)
  min.max.min$disp.b.2 <- sqrt((xb[min.max.min$t3] - xb[min.max.min$t2])^2 + (yb[min.max.min$t3] - yb[min.max.min$t2])^2)
  
  #the leader is the one who moves more during the first time interval, the follower the one that moves less
  min.max.min$leader <- a
  min.max.min$follower <- b
  min.max.min$leader[which(min.max.min$disp.b.1 > min.max.min$disp.a.1)] <- b
  min.max.min$follower[which(min.max.min$disp.b.1 > min.max.min$disp.a.1)] <- a
  
  #it's a pull if the leader moves less during the second time interval
  min.max.min$type <- 'pull'
  min.max.min$type[which((min.max.min$leader == a) & (min.max.min$disp.a.2 > min.max.min$disp.b.2))] <- 'anchor'
  min.max.min$type[which((min.max.min$leader == b) & (min.max.min$disp.b.2 > min.max.min$disp.a.2))] <- 'anchor'
  
  #get the disparity of each event
  min.max.min$disparity <- (abs(min.max.min$disp.a.1 - min.max.min$disp.b.1) * abs(min.max.min$disp.a.2 - min.max.min$disp.b.2))/((min.max.min$disp.a.1 + min.max.min$disp.b.1)*(min.max.min$disp.a.2 + min.max.min$disp.b.2))
  
  #get the strength of each event
  min.max.min$strength <- (abs(dyad.dist[min.max.min$t2] - dyad.dist[min.max.min$t1])*abs(dyad.dist[min.max.min$t3] - dyad.dist[min.max.min$t2]))/((dyad.dist[min.max.min$t2] + dyad.dist[min.max.min$t1])*(dyad.dist[min.max.min$t2] + dyad.dist[min.max.min$t3]))
  
  #remove un-needed columns
  events <- min.max.min[,c(1,2,3,8,9,10,11,12)]
  
  return(events)
  
}

get.dyad.interactions.all <- function(xa,xb,ya,yb,a,b,breaks,noise.thresh = 5, min.seq.length = 60){
  
  #check that xa, xb, ya, and yb are all the same length
  n.times <- length(xa)
  if(length(xb)!=n.times | length(ya)!=n.times | length(yb)!=n.times){
    stop('ERROR: input x and y data vectors are not all the same length')
  }
  
  #get events for each sequence, broken up by 'breaks' and any NAs
  events <- data.frame(NULL)
  for(i in 1:(length(breaks)-1)){
    begin <- breaks[i]
    finish <- breaks[i+1]-1
    xa.curr <- xa[begin:finish]
    xb.curr <- xb[begin:finish]
    ya.curr <- ya[begin:finish]
    yb.curr <- yb[begin:finish]
    
    #get indexes to NAs
    nas <- which(is.na(xa.curr) | is.na(xb.curr) | is.na(ya.curr) | is.na(yb.curr))
    nas <- c(0,nas,length(xa.curr)+1)
    
    na.diff <- diff(nas)
    
    starts <- nas[which(na.diff >= min.seq.length)]+1
    ends <- nas[which(na.diff >= min.seq.length)+1]-1
    
    
    if(length(starts)>0){
      for(j in 1:length(starts)){
        
        #get data to run analysis on
        xa.tmp <- xa.curr[starts[j]:ends[j]]
        xb.tmp <- xb.curr[starts[j]:ends[j]]
        ya.tmp <- ya.curr[starts[j]:ends[j]]
        yb.tmp <- yb.curr[starts[j]:ends[j]]
        
        #get dyadic interactions
        events.tmp <- get.dyad.interactions(xa.tmp,xb.tmp,ya.tmp,yb.tmp,a,b,noise.thresh=noise.thresh)
        
        if(!is.null(events.tmp)){
          #make times match up to the original sequence times
          events.tmp$t1 <- events.tmp$t1 + starts[j] + begin - 2
          events.tmp$t2 <- events.tmp$t2 + starts[j] + begin - 2
          events.tmp$t3 <- events.tmp$t3 + starts[j] + begin - 2
          
          #add current events to the data frame of events
          events.tmp$day <- i
          events.tmp$noise.thresh <- noise.thresh
          events <- rbind(events,events.tmp)
        }	
      }
    }
    
  }
  
  return(events)
  
}

get.all.interactions<-function(xs,ys,breaks,noise.thresh=5,min.seq.length=60){
  N <- dim(xs)[1] # Dimension of matrix columns - individuals
  t <- dim(xs)[2] # Dimension of matrix rows - times
  
  if(dim(ys)[1] != N | dim(ys)[2] != t){
    stop('ERROR: x and y matrices must have the same dimensions')
  }
  
  if(N >= t){
    warning('WARNING: number of positions per individual is greater than number of individuals - do you need to transpose your x and y matrices?')
  }
  
  events <- data.frame(NULL)
  for(a in 1:(N-1)){
    for(b in (a+1):N){
      print(a)
      print(b)
      xa <- xs[a,]
      xb <- xs[b,]
      ya <- ys[a,]
      yb <- ys[b,]
      
      events.tmp <- get.dyad.interactions.all(xa,xb,ya,yb,a,b,breaks,noise.thresh,min.seq.length)
      
      if(!is.null(events.tmp)){
        events <- rbind(events,events.tmp)
      }
      
    }
  }
  return(events)
  
}

# Da Gama GPS dataset is subset for > 10 active collars present in either urban or natural space
# and for between 06:00-16:00 UTC (remove final two hours: see manuscript)

# get.all.interactions() expects two N x T matrices for x and y coords (xs and ys respectively), where N is the number of individuals tracked, and T the number of data points
# and an argument - breaks - which is a vector containing column numbers with where to index the data
# Here, breaks is set by both day (first daily timestamp) and area (timestamp at which baboons move between natural and urban space)

min.seq.length <- 60
noise.thresh <- 5 
min.disparity <- 0.1
min.strength <- 0.1

events <- get.all.interactions(xs,ys,breaks,noise.thresh,min.seq.length)

events <- events[which(events$disparity >= min.disparity & events$strength >= min.strength),]

colnames(events)[which(names(events) == "day")] <- "day_area"

# remove duplicated timestamps (i.e. for multiple ID's) from original GPS dataset,
# and merge with "breaks" - to have information on area for each break point.
# merge with events to have "area" column

GPS <- GPS[order(GPS$time),]

GPS <- GPS[!duplicated(GPS$time),]

GPS$rowID <- 1:nrow(GPS)

breaks <- as.data.frame(breaks)
breaks$day_area <- 1:nrow(breaks)

GPS <- merge(breaks, GPS[c("rowID", "area")], by.x="breaks", by.y = "rowID", all.x=T )

events3 <- merge(events, GPS[c("day_area", "area")], by = "day_area", all.x=T)

events4 <- events3[,c("leader", "follower", "type", "area")]

# Subset dataframe for "pulls"

pulls <- events4[events4$type == "pull",]

# Create separate matrices for urban and natural

pulls_urban <- pulls[pulls$area == "Urban",]
pulls_natural <- pulls[pulls$area == "Natural",]

pulls_urban$area = NULL
pulls_natural$area = NULL

pulls_urban$leader <- as.factor(pulls_urban$leader)
pulls_urban$follower <- as.factor(pulls_urban$follower)
pulls_urban$type <- as.factor(pulls_urban$type)
pulls_urban$type <- as.numeric(pulls_urban$type)
pulls_urban <- setDT(pulls_urban)
pulls_urban <- dcast(pulls_urban, leader  ~ follower, value.var = "type", fun.aggregate = sum)
pulls_urban <- textshape::column_to_rownames(pulls_urban, 'leader')
pulls_urban <- as.matrix(pulls_urban)

pulls_natural$leader <- as.factor(pulls_natural$leader)
pulls_natural$follower <- as.factor(pulls_natural$follower)
pulls_natural$type <- as.factor(pulls_natural$type)
pulls_natural$type <- as.numeric(pulls_natural$type)
pulls_natural <- setDT(pulls_natural)
pulls_natural <- dcast(pulls_natural, leader  ~ follower, value.var = "type", fun.aggregate = sum)
pulls_natural <- textshape::column_to_rownames(pulls_natural, 'leader')
pulls_natural <- as.matrix(pulls_natural)

cor.test(c(as.matrix(pulls_natural)), c(as.matrix(pulls_urban)), method = "spearman")

# Create natural area graph and extract centrality

g_natural<- graph.adjacency(pulls_natural, 'directed',
                            diag = FALSE, weighted = TRUE)

natural_leaderNetwork <- data.table(
  centrality = evcent(g_natural)$vector,
  ID = names(degree(g_natural))
)

# Create urban area graph and extract centrality

g_urban <- graph.adjacency(pulls_urban, 'directed',
                           diag = FALSE, weighted = TRUE)

urban_leaderNetwork <- data.table(
  centrality = evcent(g_urban)$vector,
  ID = names(degree(g_urban))
)

edge_density(g_natural)
edge_density(g_urban)

natural_leaderNetwork$area = "Natural"
urban_leaderNetwork$area = "Urban"

DF <- rbind(natural_leaderNetwork, urban_leaderNetwork)

LeaderNetwork <- as.data.frame(DF)

str(LeaderNetwork)

LeaderNetwork$ID <- revalue(LeaderNetwork$ID, c("1" = "F15", "2" = "F18", "3" = "F13", "4" = "F2", "5" = "M1", "6" = "F6", "7" = "F14", "8" = "F5", "9" = "F7", "10" = "F9", "11" = "M2", "12" = "F17", "13" = "F10"))

colnames(LeaderNetwork)[which(names(LeaderNetwork) == "centrality")] <- "LeaderCentrality"


